@@ -1,14 +1,21 @@
 // Template functions
 //
-// 01/02/2004
-// 08/03/2004: cumsum added
+// 01/02/2004: 'printArray'
+//             'changePointers'
+//             'writeToFile'
+//             'writeAddToFile'
+//             'writeTwoToFile'
+//             'writeToFile2'
+//             'writeRaggedToFile'
+//             'readFromFile'
+// 08/03/2004: 'cumsum'
 //
 #ifndef TEMPLATE_FUN_CPP
 #define TEMPLATE_FUN_CPP
 
 #include "templatefun.h"
 
-using namespace std;
+// using namespace std;
 
 // ====================================================
 // Function to compute cumulative sums of an array
@@ -18,7 +25,7 @@ using namespace std;
 // vals ...... original values
 // kP ........ length of arrays csum and vals
 //
-template <typename dd>
+ template <typename dd>
 void
 cumsum(dd* csum,   const dd* vals,   const int* kP)
 {
@@ -35,12 +42,12 @@ cumsum(dd* csum,   const dd* vals,   const int* kP)
 // Function to print an array on a screen
 //  -> useful especially for debugging
 // ================================================================
-template <typename pp>
+ template <typename pp>
 void
 printArray(const pp* array, const int* length)
 {
-  for (int i = 0; i < *length; i++) cout << array[i] << ",  ";
-  cout << endl;
+  for (int i = 0; i < *length; i++) std::cout << array[i] << ",  ";
+  std::cout << std::endl;
   return;
 }
 
@@ -48,7 +55,7 @@ printArray(const pp* array, const int* length)
 // ================================================================================
 // Function to change two pointers
 // ================================================================================
-template <typename pp>
+ template <typename pp>
 void
 changePointers(pp** aP, pp** bP)
 {
@@ -61,7 +68,7 @@ changePointers(pp** aP, pp** bP)
 
 
 // ================================================================================
-// Function to write a numeric array to a file
+// ***** writeToFile: Function to write a numeric array to a file             *****
 // ================================================================================
 // The content of the array is written to the file in ROW major order.
 //
@@ -71,43 +78,16 @@ changePointers(pp** aP, pp** bP)
 //                         'o': owerwrite
 //                         'n': don't replace
 //
-template <typename dd>
+ template <typename dd>
 void
-writeToFile(const dd* array,          const int nR,                 const int nC,
-            const std::string& dir,   const std::string& filename,  const char &flag,
+writeToFile(const dd* array,          const int& nR,                const int& nC,
+            const std::string& dir,   const std::string& filename,  const char& flag,
             const int& prec,          const int& width)
 {
   try{
     std::string path = dir + filename;
     std::ofstream out;
-    bool err = false;
-
-    std::string errmess;
-    if (flag == 'n') {
-      std::fstream temp(path.c_str(), std::ios::in);
-      if (!temp) {
-	out.open(path.c_str(), std::ios::out);
-      } 
-      else {
-	temp.close();
-	err = true;
-      }
-    }
-    else if (flag == 'o')
-           out.open(path.c_str(), std::ios::out | std::ios::trunc);
-         else if (flag == 'a')
-                out.open(path.c_str(), std::ios::out | std::ios::app);
-              else {
-                errmess = std::string("C++ Error: Incorrect flag for writing to ") + path + ". ";
-		returnR error(errmess, 99);
-                throw error;
-              }
-
-    if (!out || err) {
-      errmess = std::string("C++ Error: Could not open ") + path + " for writing. ";
-      returnR error(errmess, 99);
-      throw error; 
-    }
+    openFile(out, path, flag);
 
     std::ostringstream s;
     unsigned int mlen = width;
@@ -149,11 +129,11 @@ writeToFile(const dd* array,          const int nR,                 const int nC
           }
         }
       }
-      out << endl;
+      out << std::endl;
     }
 //    out << s.str();
     out.close();
-
+    return;
   }  // end of try
   catch(returnR){
     throw;
@@ -161,6 +141,83 @@ writeToFile(const dd* array,          const int nR,                 const int nC
 }   // end of function writeToFile
 
 
+// ================================================================================
+// ***** writeAddToFile: Function to write a numeric array to a file          *****
+//         - some value is added to all elements of the array before
+//           writting them to the file
+//         - useful when recording C++/R indeces
+// ================================================================================
+// The content of the array is written to the file in ROW major order.
+//
+// nR .......... number of rows into which the array should be splitted
+// nC .......... number of columns into which the array should be splitted 
+// flag ........ it can be 'a': append
+//                         'o': owerwrite
+//                         'n': don't replace
+//
+ template <typename dd>
+void
+writeAddToFile(const dd* array,          const int& nR,                const int& nC,     const dd& add,
+               const std::string& dir,   const std::string& filename,  const char& flag,
+               const int& prec,          const int& width)
+{
+  try{
+    std::string path = dir + filename;
+    std::ofstream out;
+    openFile(out, path, flag);
+
+    std::ostringstream s;
+    unsigned int mlen = width;
+
+    /* Passes up to 5 rows to get things to line up nicely */
+    for (int i = 0; i < nR && i < 5; i++) {
+      for (int j = 0; j < nC; j++) {
+	s.str("");        
+        if (array[i*nC + j] + add >= FLT_MAX){
+	  s << std::setw(width)
+	    << std::setiosflags(std::ios::fixed)
+	    << "1e50" << "   ";
+        }
+        else{
+          if (array[i*nC + j] + add < 1 && array[i*nC + j] + add > -1) 
+            s << std::scientific << std::setw(width) << std::setprecision(prec) << array[i*nC + j] + add << "   ";
+          else
+            s << std::fixed << std::setw(width) << std::setprecision(prec) << array[i*nC + j] + add << "   ";
+        }
+	if (s.str().length() > mlen) mlen = s.str().length();
+      }
+    }
+
+//    s.str("");
+    for (int i = 0; i < nR; i++) {
+      for (int j = 0; j < nC; j++){
+        if (array[i*nC + j] + add >= FLT_MAX){
+          out << std::setw(mlen) << "1e50";
+          out << "   ";
+	}
+        else{
+          if (array[i*nC + j] + add < 1 && array[i*nC + j] + add > -1){
+            out << std::scientific << std::setw(mlen) << std::setprecision(prec) << array[i*nC + j] + add;
+            out << "   ";
+          }
+          else{
+            out << std::fixed << std::setw(mlen) << std::setprecision(prec) << array[i*nC + j] + add;
+            out << "   ";
+          }
+        }
+      }
+      out << std::endl;
+    }
+//    out << s.str();
+    out.close();
+    return;
+  }  // end of try
+  catch(returnR){
+    throw;
+  }  
+}   // end of function writeAddToFile
+
+
 // ================================================================================
 // Function to write part of one array and whole second array to a file
 // ================================================================================
@@ -176,45 +233,18 @@ writeToFile(const dd* array,          const int nR,                 const int nC
 //                          'o': owerwrite
 //                          'n': don't replace
 //
-template <typename dd>
+ template <typename dd1, typename dd2>
 void
-writeTwoToFile(const dd* array1,         const int nR1,                 const int nC1,      const int col1,
-               const dd* array2,         const int nR2,                 const int nC2,
-               const std::string& dir,   const std::string& filename,   const char &flag,
+writeTwoToFile(const dd1* array1,        const int& nR1,                 const int& nC1,      const int& col1,
+               const dd2* array2,        const int& nR2,                 const int& nC2,
+               const std::string& dir,   const std::string& filename,    const char &flag,
                const int& prec,          const int& width)
 {
   try{
     if (nR1 != nR2) throw returnR("C++ programming error: contact the author", 99);
     std::string path = dir + filename;
     std::ofstream out;
-    bool err = false;
-
-    std::string errmess;
-    if (flag == 'n') {
-      std::fstream temp(path.c_str(), std::ios::in);
-      if (!temp) {
-	out.open(path.c_str(), std::ios::out);
-      } 
-      else {
-	temp.close();
-	err = true;
-      }
-    }
-    else if (flag == 'o')
-           out.open(path.c_str(), std::ios::out | std::ios::trunc);
-         else if (flag == 'a')
-                out.open(path.c_str(), std::ios::out | std::ios::app);
-              else {
-                errmess = std::string("C++ Error: Incorrect flag for writing to ") + path + ". ";
-		returnR error(errmess, 99);
-                throw error;
-              }
-
-    if (!out || err) {
-      errmess = std::string("C++ Error: Could not open ") + path + " for writing. ";
-      returnR error(errmess, 99);
-      throw error; 
-    }
+    openFile(out, path, flag); 
 
     std::ostringstream s;
     unsigned int mlen = width;
@@ -272,11 +302,11 @@ writeTwoToFile(const dd* array1,         const int nR1,                 const in
           }
         }
       }
-      out << endl;
+      out << std::endl;
     }
 //    out << s.str();
     out.close();
-
+    return;
   }  // end of try
   catch(returnR){
     throw;
@@ -296,7 +326,7 @@ writeTwoToFile(const dd* array1,         const int nR1,                 const in
 //                         'o': owerwrite
 //                         'n': don't replace
 //
-template <typename dd>
+ template <typename dd>
 void
 writeToFile2(dd** array,               const int n1,                 const int n2,
              const std::string& dir,   const std::string& filename,  const char &flag,
@@ -305,34 +335,7 @@ writeToFile2(dd** array,               const int n1,                 const int n
   try{
     std::string path = dir + filename;
     std::ofstream out;
-    bool err = false;
-
-    std::string errmess;
-    if (flag == 'n') {
-      std::fstream temp(path.c_str(), std::ios::in);
-      if (!temp) {
-	out.open(path.c_str(), std::ios::out);
-      } 
-      else {
-	temp.close();
-	err = true;
-      }
-    }
-    else if (flag == 'o')
-           out.open(path.c_str(), std::ios::out | std::ios::trunc);
-         else if (flag == 'a')
-                out.open(path.c_str(), std::ios::out | std::ios::app);
-              else {
-                errmess = std::string("C++ Error: Incorrect flag for writing to ") + path + ". ";
-		returnR error(errmess, 99);
-                throw error;
-              }
-
-    if (!out || err) {
-      errmess = std::string("C++ Error: Could not open ") + path + " for writing. ";
-      returnR error(errmess, 99);
-      throw error; 
-    }
+    openFile(out, path, flag); 
 
     std::ostringstream s;
     unsigned int mlen = width;
@@ -374,11 +377,11 @@ writeToFile2(dd** array,               const int n1,                 const int n
           }
         }
       }
-      out << endl;
+      out << std::endl;
     }
 //    out << s.str();
     out.close();
-
+    return;
   }  // end of try
   catch(returnR){
     throw;
@@ -386,31 +389,123 @@ writeToFile2(dd** array,               const int n1,                 const int n
 }   // end of function writeToFile2
 
 
+// ===========================================
+// Function to write a ragged array to file
+// ===========================================
+// 
+// array[nR*maxnC] ..... array to be written to a file
+//                       * values stored in ROW MAJOR order
+//                       * this array is assumed to be regular (not ragged),
+//                         however, maxnC - multnC * nC[i] from the i-th row will be ignored for writting
+// nR .................. number of rows into which the array is to be splitted
+// maxnC ............... maximal number of columns (i.e. nC[j]*multnC <= maxnC for all j)
+// nC[nR]............... number of columns on each row that should be after multiplying by multnC written to the file
+// multnC .............. each nC is multiplied by multnC to get number of columns that should be written to the file
+//                       (useful for writing bivariate (multnc = 2) means, e.g. in bayesHistogram)
+//
+ template <typename dd>
+void
+writeRaggedToFile(const dd* array,         const int& nR,                const int& maxnC,  
+                  const int* nC,           const int& multnC,
+                  const std::string& dir,  const std::string& filename,  const char &flag,
+                  const int& prec,         const int& width)
+{
+  try{
+    int i, j;
+    std::string path = dir + filename;
+    std::ofstream out;
+    openFile(out, path, flag);
+
+    /*** Write to the file ***/
+    std::ostringstream s;
+    unsigned int mlen = width;
+   
+    /* Passes up to 5 rows to get things to line up nicely */
+    for (i = 0; i < nR && i < 5; i++) {
+      if (multnC * nC[i] > maxnC) throw returnR("C++ Error: multnC * nC must be <= maxnC in writeRaggedToFile", 1);
+      for (j = 0; j < multnC * nC[i]; j++) {
+	s.str("");        
+        if (array[i*maxnC + j] >= FLT_MAX){
+	  s << std::setw(width)
+	    << std::setiosflags(std::ios::fixed)
+	    << "1e50" << "   ";
+        }
+        else{
+          if (array[i*maxnC + j] < 1 && array[i*maxnC + j] > -1) 
+            s << std::scientific << std::setw(width) << std::setprecision(prec) << array[i*maxnC + j] << "   ";
+          else
+            s << std::fixed << std::setw(width) << std::setprecision(prec) << array[i*maxnC + j] << "   ";
+        }
+	if (s.str().length() > mlen) mlen = s.str().length();
+      }
+    }
+
+    /* Write */
+//    s.str("");
+    for (i = 0; i < nR; i++) {
+      if (multnC * nC[i] > maxnC) throw returnR("C++ Error: multnC * nC must be <= maxnC in writeRaggedToFile", 1);
+      for (j = 0; j < multnC * nC[i]; j++){
+        if (array[i*maxnC + j] >= FLT_MAX){
+          out << std::setw(mlen) << "1e50";
+          out << "   ";
+	}
+        else{
+          if (array[i*maxnC + j] < 1 && array[i*maxnC + j] > -1){
+            out << std::scientific << std::setw(mlen) << std::setprecision(prec) << array[i*maxnC + j];
+            out << "   ";
+          }
+          else{
+            out << std::fixed << std::setw(mlen) << std::setprecision(prec) << array[i*maxnC + j];
+            out << "   ";
+          }
+        }
+      }
+      out << std::endl;
+    }
+//    out << s.str();
+    out.close();
+    return;  
+  }
+  catch(returnR){
+    throw;
+  }  
+}  /** end of function writeRaggedToFile **/
+
+
 
 // =========================================
 // Function to read an array from a file
 // =========================================
 // The file is read by ROWS and written to an array
 //
-// array........ array where to read it        (length = nR * nC)
-// nR .......... number of rows in file THAT ARE TO BE READ
+// array........ array where to read it        (length = nread * nC)
+// nread ....... on OUTPUT: number of read rows
+// nR .......... number of rows in file THAT ARE AVAILABLE for reading after skipping possible header
+//               (this will usually be MCMC sample size)
 // nC .......... number of columns in file THAT ARE TO BE READ
+// header ...... 0/1 is there header?
+// skip ........ how many rows at the beginning of the file are to be skipped (do not count header)
+// by .......... only every by-th mixture will be read
 // dir ......... directory
 // filename .... name of the file
 // skipOnRow ... number of values that are to be skipped at the beginning of each row
 // skip ........ number of rows thar are to be skipped at the beginnig of the file
 //
-template <typename dd>
+ template <typename dd>
 void
-readFromFile(dd* array,               const int nR,                 const int nC,
+readFromFile(dd* array,               int* nread,       
+             const int& nR,           const int& nC,               const int& header,
+             const int& skip,         const int& by,
              const std::string& dir,  const std::string& filename,  
-             const int skipOnRow,     const int skip)
+             const int& skipOnRow)
 {
   try{
+    int i, j, ii;
     int size = nR * nC;
-    if (size <= 0){
-      throw returnR("C++ Error: File of null size is to be read.", 99);
-    }
+    if (size <= 0) throw returnR("C++ Error: File of null size is to be read.", 99);
+    if (skip < 0) throw returnR("C++ Error: 'skip' parameter must be >= 0 in 'readFromFile'", 1);
+    if (by <= 0) throw returnR("C++ Error: 'by' parameter must be > 0 in 'readFromFile'", 1);
+    if (skip >= nR) throw returnR("C++ Error: too many rows are to be skipped by 'readFromFile'", 1);
 
     std::string path = dir + filename;
 
@@ -427,36 +522,74 @@ readFromFile(dd* array,               const int nR,                 const int nC
       mess = std::string("Reading ") + path + "\n";
       strcpy(cmess, mess.c_str());
       Rprintf(cmess);
-//      cout << mess;
-      int k = 0;
+
+      /*** Skip what is to be skipped (header included) ***/
       char ch;
-      for (int i = 0; i < skip; i++){
-        file.get(ch);
+      for (i = 0; i < skip + header; i++){
+        file.get(ch);        
         while (ch != '\n') file.get(ch);
       }
-      for (int i = 0; i < nR; i++){
-        for (int j = 0; j < skipOnRow; j++){
+
+      /*** Read the first row to be read ***/
+      *nread = 1;
+      double* veld = array;
+      if (file.eof()){
+        errmes = std::string("C++ Error: Reached end of file ") + path + std::string(" before ") 
+                 + char(*nread) + std::string(" rows were read.");
+        throw returnR(errmes, 99);
+      }
+      for (j = 0; j < skipOnRow; j++){
+        if (file.eof()){
+          errmes = std::string("C++ Error: Reached end of file ") + path + std::string(" before ") 
+                   + char(*nread) + std::string(" rows were read.");
+          throw returnR(errmes, 99);
+        }
+        file >> temp;
+      }
+      for (j = skipOnRow; j < nC + skipOnRow; j++){
+        if (file.eof()){
+          errmes = std::string("C++ Error: Reached end of file ") + path + " before "
+                   + char(*nread) + std::string(" rows were read.");
+          throw returnR(errmes, 99);
+        }
+        file >> (*veld);
+        veld++;
+      }
+
+      /*** Read remaining rows to be read ***/
+      for (i = skip + 1 + by; i <= nR; i += by){
+
+        /** Skip by-1 rows **/
+        for (ii = 0; ii < by - 1; ii++){
+          file.get(ch);        
+          while (ch != '\n') file.get(ch);
+        }
+
+        /** Read the values **/
+        (*nread)++;
+        for (j = 0; j < skipOnRow; j++){
           if (file.eof()){
             errmes = std::string("C++ Error: Reached end of file ") + path + std::string(" before ") 
-                     + char(size) + std::string(" values were read.");
+                     + char(*nread) + std::string(" rows were read.");
             throw returnR(errmes, 99);
           }
           file >> temp;
         }
-        for (int j = skipOnRow; j < nC + skipOnRow; j++){
+        for (j = skipOnRow; j < nC + skipOnRow; j++){
           if (file.eof()){
             errmes = std::string("C++ Error: Reached end of file ") + path + " before "
-                     + char(size) + std::string(" values were read.");
+                     + char(*nread) + std::string(" rows were read.");
             throw returnR(errmes, 99);
           }
-          file >> array[k];
-          k++;
+          file >> (*veld);
+          veld++;
         }
+        file.get(ch);                 
+        while (ch != '\n') file.get(ch);
       }
     }
     file.close();
     return;
-
   }  // end of try
   catch(returnR){
     throw;
